@@ -7,8 +7,10 @@ pub use self::offset_page_table::OffsetPageTable;
 pub use self::recursive_page_table::{InvalidPageTable, RecursivePageTable};
 
 use crate::structures::paging::{
-    frame_alloc::FrameAllocator, page_table::PageTableFlags, Page, PageSize, PhysFrame, Size1GiB,
-    Size2MiB, Size4KiB,
+    frame_alloc::{FrameAllocator, FrameDeallocator},
+    page::PageRangeInclusive,
+    page_table::PageTableFlags,
+    Page, PageSize, PhysFrame, Size1GiB, Size2MiB, Size4KiB,
 };
 use crate::{PhysAddr, VirtAddr};
 
@@ -85,25 +87,21 @@ pub enum MappedFrame {
 }
 
 impl MappedFrame {
-    const_fn! {
-        /// Returns the start address of the frame.
-        pub fn start_address(&self) -> PhysAddr {
-            match self {
-                MappedFrame::Size4KiB(frame) => frame.start_address(),
-                MappedFrame::Size2MiB(frame) => frame.start_address(),
-                MappedFrame::Size1GiB(frame) => frame.start_address(),
-            }
+    /// Returns the start address of the frame.
+    pub const fn start_address(&self) -> PhysAddr {
+        match self {
+            MappedFrame::Size4KiB(frame) => frame.start_address,
+            MappedFrame::Size2MiB(frame) => frame.start_address,
+            MappedFrame::Size1GiB(frame) => frame.start_address,
         }
     }
 
-    const_fn! {
-        /// Returns the size the frame (4KB, 2MB or 1GB).
-        pub fn size(&self) -> u64 {
-            match self {
-                MappedFrame::Size4KiB(frame) => frame.size(),
-                MappedFrame::Size2MiB(frame) => frame.size(),
-                MappedFrame::Size1GiB(frame) => frame.size(),
-            }
+    /// Returns the size the frame (4KB, 2MB or 1GB).
+    pub const fn size(&self) -> u64 {
+        match self {
+            MappedFrame::Size4KiB(_) => Size4KiB::SIZE,
+            MappedFrame::Size2MiB(_) => Size2MiB::SIZE,
+            MappedFrame::Size1GiB(_) => Size1GiB::SIZE,
         }
     }
 }
@@ -385,8 +383,11 @@ pub struct MapperFlush<S: PageSize>(Page<S>);
 
 impl<S: PageSize> MapperFlush<S> {
     /// Create a new flush promise
+    ///
+    /// Note that this method is intended for implementing the [`Mapper`] trait and no other uses
+    /// are expected.
     #[inline]
-    fn new(page: Page<S>) -> Self {
+    pub fn new(page: Page<S>) -> Self {
         MapperFlush(page)
     }
 
@@ -407,14 +408,17 @@ impl<S: PageSize> MapperFlush<S> {
 /// The old mapping might be still cached in the translation lookaside buffer (TLB), so it needs
 /// to be flushed from the TLB before it's accessed. This type is returned from a function that
 /// made the change to ensure that the TLB flush is not forgotten.
-#[derive(Debug)]
+#[derive(Debug, Default)]
 #[must_use = "Page Table changes must be flushed or ignored."]
 pub struct MapperFlushAll(());
 
 impl MapperFlushAll {
     /// Create a new flush promise
+    ///
+    /// Note that this method is intended for implementing the [`Mapper`] trait and no other uses
+    /// are expected.
     #[inline]
-    fn new() -> Self {
+    pub fn new() -> Self {
         MapperFlushAll(())
     }
 
@@ -478,3 +482,45 @@ pub enum TranslateError {
 }
 
 static _ASSERT_OBJECT_SAFE: Option<&(dyn Translate + Sync)> = None;
+
+/// Provides methods for cleaning up unused entries.
+pub trait CleanUp {
+    /// Remove all empty P1-P3 tables
+    ///
+    /// ## Safety
+    ///
+    /// The caller has to guarantee that it's safe to free page table frames:
+    /// All page table frames must only be used once and only in this page table
+    /// (e.g. no reference counted page tables or reusing the same page tables for different virtual addresses ranges in the same page table).
+    unsafe fn clean_up<D>(&mut self, frame_deallocator: &mut D)
+    where
+        D: FrameDeallocator<Size4KiB>;
+
+    /// Remove all empty P1-P3 tables in a certain range
+    /// ```
+    /// # use core::ops::RangeInclusive;
+    /// # use x86_64::{VirtAddr, structures::paging::{
+    /// #    FrameDeallocator, Size4KiB, MappedPageTable, mapper::{RecursivePageTable, CleanUp}, page::{Page, PageRangeInclusive},
+    /// # }};
+    /// # unsafe fn test(page_table: &mut RecursivePageTable, frame_deallocator: &mut impl FrameDeallocator<Size4KiB>) {
+    /// // clean up all page tables in the lower half of the address space
+    /// let lower_half = Page::range_inclusive(
+    ///     Page::containing_address(VirtAddr::new(0)),
+    ///     Page::containing_address(VirtAddr::new(0x0000_7fff_ffff_ffff)),
+    /// );
+    /// page_table.clean_up_addr_range(lower_half, frame_deallocator);
+    /// # }
+    /// ```
+    ///
+    /// ## Safety
+    ///
+    /// The caller has to guarantee that it's safe to free page table frames:
+    /// All page table frames must only be used once and only in this page table
+    /// (e.g. no reference counted page tables or reusing the same page tables for different virtual addresses ranges in the same page table).
+    unsafe fn clean_up_addr_range<D>(
+        &mut self,
+        range: PageRangeInclusive,
+        frame_deallocator: &mut D,
+    ) where
+        D: FrameDeallocator<Size4KiB>;
+}
